#!/usr/bin/env python
# -*- coding: utf-8 -*-

import gtk
import gtk.gdk
import pango

import sys, warnings

from liboogtk.gtk_dicos import *
import libvisioo.oostyles as oostyles

class ZoneTexte (gtk.TextView):
	"""
	gtk.Textview with simple new methods
	fr : crèe une instance de type gtk.TextView avec qulques nouvelles méthodes

	Exemple d'utilisation
	win=gtk.Window()
	zone=Zonetexte()
	zone.bordure(10,10,10,10)
	zone.texte("Presentation style gtk", [("size_points",20)])
	zone.texte_ed("\\g\\uPresentation \\cformatee")
	zone.image("chemin")
	zone.objet(gtk.Button("ok"))
	win.add(zone)

	Variables
	self.oobuffer : gtk.TextBuffer
	self.table : gtk.TextTagTable
	self. : gtk.TextView

	"""

	def __init__ (self, buffer_name='main'):

		gtk.TextView.__init__(self)
		self.table = gtk.TextTagTable()

		self.oobuffer = gtk.TextBuffer(self.table)

		self.set_buffer(self.oobuffer)
		self.set_wrap_mode(gtk.WRAP_WORD)
		self.set_editable(False)
		self.set_cursor_visible(False)

		self.fin = self.oobuffer.get_start_iter()

		self.paragraph_new = True
		self.list_new = True

		#styles texte_ed (encoded simplified text)
		self.color = "#0000FF"
		self.underline = 1
		self.justification = gtk.JUSTIFY_CENTER

	def close (self) :
		pass

	def config (self, marges = (10,10), interligne = 0):
		"""
		marges (type = tuple of int)
		interligne (type = int)
		configuration marges and interline
		"""
		self.set_left_margin(marges[0])
		self.set_right_margin(marges[1])
		self.set_pixels_above_lines(interligne)

	def bordure (self, haut, droite, bas, gauche):
		""" configuration borders
		haut : top (type=int)
		droite : right (type=int)
		bas : bottom (type=int)
		gauche : left (type=int)
		"""
		self.set_border_window_size(gtk.TEXT_WINDOW_RIGHT, droite)
		self.set_border_window_size(gtk.TEXT_WINDOW_LEFT, gauche)
		self.set_border_window_size(gtk.TEXT_WINDOW_TOP, haut)
		self.set_border_window_size(gtk.TEXT_WINDOW_BOTTOM, bas)

	def def_wrap_mode (self, mode) :
		"""
		mode : type of wrapping (type=str)
		ex : "WRAP_NONE" (see gtk_dicos)
		"""
		self.set_wrap_mode(DICO_PARAM[mode])

	def clean (self):
		"""
		empty buffer
		fr : vide le buffer
		"""
		self.oobuffer.remove_all_tags(self.oobuffer.get_start_iter(), self.oobuffer.get_end_iter())
		self.oobuffer.delete(self.oobuffer.get_start_iter(), self.oobuffer.get_end_iter())
		self.fin = self.oobuffer.get_start_iter()

	def not_empty (self) :
		"""
		return 0 if buffer is empty, else return integer
		fr : renvoie 0 si le buffer est vide, 1 entier sinon
		"""
		return self.oobuffer.get_char_count()

	def get_line_count (self) :
		"""
		return number of lines in buffer
		fr : renvoie le nombre de lignes du buffer
		"""
		return self.oobuffer.get_line_count()

	def get_char_count (self) :
		""" return number of caracters in buffer
		fr : renvoie le nombre de caractères du buffer
		"""
		return self.oobuffer.get_char_count()

	def image_abs (self, path):
		"""
		display the picture at path at the end of buffer
		fr : affiche l'image definie par path (chemin absolu) à la fin du buffer
		>>> ZoneTexte.image_abs("/home/guillaume/test.jpg")
		"""
		path = os.path.normpath(path)
		img = gtk.gdk.pixbuf_new_from_file(path)
		self.oobuffer.insert_pixbuf(self.oobuffer.get_end_iter(), img)

	def image (self, fichier):
		"""
		display at end of buffer the picture fichier.
		fichier must be in the extract dir
		fr : affiche en fin de buffer l'image fichier.
		L'image doit être dans le dossier d'extraction.

		>>> ZoneTexte.image("Pictures/0065306036653.png")
		"""
		fichier = os.path.normpath(fichier)
		img = gtk.gdk.pixbuf_new_from_file(TEMP_EXTRACT_DIR+sep+fichier)
		self.oobuffer.insert_pixbuf(self.oobuffer.get_end_iter(), img)


	def objet_abs (self, zone, objet, x, y):
		"""
		zones gtk :
		gtk.TEXT_WINDOW_RIGHT
		gtk.TEXT_WINDOW_LEFT
		gtk.TEXT_WINDOW_TOP
		gtk.TEXT_WINDOW_BOTTOM

		objet (type=gtk.Object)
		x,y (type=int)

		display objet (object) at x, y
		fr : affiche l'objet dans la zone aux coordonnées x,y

		>>> ZoneTexte.objet_abs (gtk.TEXT_WINDOW_TOP, widget, 10, 20)

		"""
		self.add_child_in_window(objet, zone, x, y)

	def objet (self, objet) :
		"""
		objet : gtk.Object (type=gtk.Object)
		display objet at end of buffer
		fr : affiche l'objet à la fin du buffer
		"""
		ancre = self.oobuffer.create_child_anchor(self.oobuffer.get_end_iter())
		self.add_child_at_anchor(objet, ancre)
		self.texte("\n")
		self.texte("\n")

	def set_puce (self, style, num):
		"""
		english name would be : set_bullet

		display a bullet
		fr : affiche une puce

		Note tmp : affiche un tab puis l'image default du dossier puces et ajoute un espace
		style, num pas encore utilisés
		"""
		self.texte("\t")
		img="%s%spuces%sdefault.png" %(PICS_PATH, sep, sep)
		self.image_abs(img)
		print "ok"
# 		self.texte(" ")


	def texte_oo (self, texte, oostyle):
		"""
		texte (type=str)
		oostyle (type=libvisioo.OOStyle)

		display var_texte with OOStyle oostyle
		fr : affiche var_texte avec le OOStyle oostyle
		"""

		nom_tag = ""

		for style in oostyle.properties :
			if style[0] in DICO_STYLE or style[1] in DICO_VALUE :
				""" if known style or value, add type->value to nom_tag"""
				nom_tag+="%s->%s ; " %(str(style[0]),str(style[1]))

			elif style[0] == "fo:break-before" : self.do_break_before (style[1])

			else : todo ("PROP : style", style[0], aff = 1)

		if not self.table.lookup(nom_tag) :
			# If no tag, adding to table

			tag = gtk.TextTag(nom_tag)
			self.table.add(tag)
			for style in oostyle.properties :
				prop = convert(style[0], style[1])
				if prop :
					prop = prop[0]
					tag.set_property(prop[0], prop[1])

		self.fin = self.oobuffer.get_end_iter()
		self.oobuffer.insert_with_tags_by_name(self.fin, texte, nom_tag)

		self.paragraph_new = False

	def texte (self, texte):
		"""
		texte (type=str)

		display raw text at the end of buffer
		fr : affiche du texte brut à la fin du buffer
		"""
		self.fin = self.oobuffer.get_end_iter()
		self.oobuffer.insert(self.fin, texte)

	def texte_ui (self, texte, liste_style = []):
		"""
		Deprecated .... DONT USE
		"""
		info = 'use texte_oo or texte instead of texte_ui'
		warnings.warn(info, DeprecationWarning, stacklevel=2)
		self.texte(texte)

	def new_paragraph (self): self.paragraph_new = True
	def new_list (self): self.list_new = True

	def is_new_paragraph (self) : return self.paragraph_new
	def is_new_list (self) : return self.list_new


	def do_break_before (self, brtype):
		"""handle page return"""
		if brtype == "page" and self.paragraph_new :

			center = oostyles.OOStyle("center")
			center.add_prop(('fo:text-align','center'))

			self.texte("\n"*5)
			self.texte_oo("-"*140, center)
			self.texte("\n"*2)

			sys.stderr.write (".") #first simple progress bar

if __name__ == "__main__" :
	pass