# -*- coding: utf-8 -*-

class OOStyle :
	"""
	Handle OpenOffice and OpenDocument Styles ::
	 fr : Gestion des styles OpenOffice et OpenDocument
	"""
	def __init__ (self, name=""):
		"""
		@param name: name of the new style
		@type name: str
		@return: None
		"""
		self.name=name #ex P1
		self.family=None #'text'

		self.list_style=[] #[<class oostyle1>, <class oostyle2>]

		self.properties=[] #[(param1, value1),(param2, value2)]
		self.attributes=[] # [param1, param2]

	def set_family (self, family):
		"""
		Set the style family (paragraph, title, ...) ::
		 fr : Definir la famille du style
		@param family: family type
		@type family: str
		"""
		self.family=family

	def get_family (self) :
		"""
		return the style family (paragraph, title, ...) ::
		 fr : retourne la famille du style
		"""
		return self.family

	def add_prop (self, property):
		"""
		add a propertie to OOStyle ::
		 fr : ajouter une proprieté au OOStyle

		 Example:
		 	('fo:font-size', '12pt')

		 see help or dicos for further informations

		@param property: odf.xml property
		@type property: (param, pvalue)
		"""
		if property[0] not in self.attributes :
			self.properties.append(property)
			self.attributes.append(property[0])
		else : self.properties[self.attributes.index(property[0])] = property

	def add_props (self, props):
		"""
		same as add_prop but value is a list of tuple whereas of add_prop
		(which is a simple tuple) ::
		 fr : comme add_prop mais value est une liste de tuple alors que
		 dans add_prop il s'agit d'un simple tuple
		@param props: list of odf.xml properties
		@type props: list of properties (param, pvalue)
		"""
		for elem in props : self.add_prop(elem)

	def add_style (self, style):
		"""
		add an OOStyle to this OOStyle ::
		 fr : ajoute un OOStyle à ce OOStyle

		@param style: the style to add to this style
		@type style: libvisioo.oostyles.OOStyle
		"""
		if not self.family : self.family = style.family
		self.list_style.append(style)
		self.name += style.name
		for elem in style.properties :
			if elem not in self.properties :
				self.add_prop(elem)

	def lookup (self, style_name) :
		"""
		Search if a style is in this OOStyle ::
		 fr : Chercher si un style appartient à ce OOStyle
		@param style_name: the name of the style to search
		@type style_name: str
		@return: True if exist, else False
		"""
		for elem in self.list_style :
			if style_name==elem.name : return True
		return False

	def get_property (self, name):
		"""
		get the coresponding value of the property "name" ::
		 fr : récuperer la valeur corespondant à la proprieté "name"
		@param name: the name of the property (ex: "fo:font-size")
		@type name: str
		"""
		for elem in self.properties :
			if elem[0]==name : return elem[1]
		return None

	def get_properties (self) :
		return self.properties

	def get_font_properties (self):
		"""
		get font informations ::
		 fr : récuperer les informations de la police de caractère

		@return: (font_name, size)
		"""
		font_name = self.get_property("style:font-name")
		size = self.get_property("fo:font-size")
		return font_name, size

	def set_name (self, name=""):
		"""
		set style name ::
		 fr : definir le nom du style

		@param name: name of the style
		@type name: str
		"""
		self.name=name

	def get_name (self) :
		"""
		return style name ::
		 fr : retourne le nom du style
		"""
		return self.name

	def __str__ (self) :
		""" display class values in an xml style """
		string = []
		string.append('<class name="OOStyle">')
		string.append('\t<param id="name" value="%s" />' %self.name)
		string.append('\t<properties number="%s">' %len(self.properties))
		for prop in self.properties :
			string.append('\t\t<param id="%s" value="%s" />' %(prop[0], prop[1]))
		string.append('\t</properties>')
		string.append('</class>')
		return '\n'.join(string)


EMPTY_STYLE=OOStyle()

if __name__ == "__main__" :
	style = OOStyle ('StyleDeTest')
	style.add_prop(('fo:font-style','italic'))
	style.add_prop(('fo:font-weight','bold'))
	print style
