# -*- coding: utf-8 -*-

import xml.sax

import libvisioo.oostyles as oostyles
import libvisioo.ootable as ootable
import libvisioo.oocell as oocell
import libvisioo.ooobject as ooobject
import libvisioo.oofichier as oofichier

#from libvisioo.locales import lg
from sxw_hdl_var import OOVar

from libvisioo.debug import *

############ Fonction principale : parse le document content.xml ############
############ Main function : parse content.xml file #########################


class Styles (OOVar, xml.sax.ContentHandler):
	"""Traitement du fichier styles.xml """
	def __init__ (self, ui_name) :
		OOVar.__init__(self, ui_name)

	def load_style (self, style):
		if style in self.dico_styles.keys() :
			return self.dico_styles[style]
		else :
			todo ("STYLE NON GERE", "", style, 1)
			return oostyles.EMPTY_STYLE

	def maj_style (self, style):
		if not self.tmp_textfinal_style.lookup(style) :
			self.tmp_textfinal_style.add_style(self.load_style(style))

	def characters (self, content):
		""" contenu de style.xml càd HEADER et FOOTER"""
		if self.TEXT_H : self.maj_style(self.tmp_texth_style)
		if self.TEXT_P : self.maj_style(self.tmp_textp_style)
		if self.TEXT_SPAN : self.maj_style(self.tmp_textspan_style)

		if self.HEADER : self.HEADER_CONTENT.texte_oo(content, self.tmp_textfinal_style)
		elif self.FOOTER : self.FOOTER_CONTENT.texte_oo(content, self.tmp_textfinal_style)

	def startElement(self, name, attrs):
		# Définition des styles
		if name=="office:styles" : self.OFFICE_STYLES=True
		elif name=="style:header" : self.HEADER=True
		elif name=="style:footer" : self.FOOTER=True
		elif name=="text:p" : self.init_textp(attrs)
		elif name=="text:span" : self.init_textspan(attrs)

		elif self.OFFICE_STYLES :
			if name=="style:style" : self.init_style_decl(attrs)
			elif name=="text:list-style" : self.init_text_list_style(attrs)
			elif self.STYLE_STYLE :
				if name=="style:properties": self.init_style_properties(attrs)
				elif name in ("style:tab-stop", "style:tab-stops") : self.init_style_tab_stop(attrs)
				else : todo("Option de style",name,attrs.keys(), True)


	def endElement(self, name):
		if name=="office:styles" : self.OFFICE_STYLES=False
		elif name=="style:header" : self.HEADER=False
		elif name=="style:footer" : self.FOOTER=False
		elif name=="office:body" : self.BODY=False
		elif name=="text:p" : self.end_textp()
		elif name=="text:span" : self.end_textspan()
		elif name=="style:style" : self.end_style_decl()
		elif name=="style:properties": self.end_style_properties()


	""" DEFINITION DES STYLES - STYLES DEFINITION"""
	""" Exemple de code xml :
		<style:style style:name = "Style 1" style:family = "paragraph" style:parent-style-name = "Text body" style:class = "text">
			<style:properties prop = "value"/>
		</style:style>
	"""
	def init_style_decl(self, attrs):
		self.STYLE_STYLE = True

		self.tmp_nom_style = attrs.get('style:name')
		self.tmp_instance_style.set_name(attrs.get('style:name'))
		self.tmp_instance_style.set_family (attrs.get('style:family'))

		parent = attrs.get('style:parent-style-name')
		next = attrs.get('style:next-style-name')

		self.tmp_instance_style.add_props(self.load_style(parent).properties)
		self.tmp_instance_style.add_props(self.load_style(next).properties)

	def init_style_properties(self, attrs):
		self.tmp_style_properties += attrs.items()
		# attrs -> [("prop1","value1), ...] ex [('fo:font-weight','normal')]

	def end_style_decl(self):
		""" update dico_style, del self.tmp_instance_style, self.tmp_nom_style """
		self.dico_styles.update({self.tmp_nom_style:self.tmp_instance_style})
		self.STYLE_STYLE = False
		self.tmp_instance_style = oostyles.OOStyle()
		self.tmp_nom_style = ""

	def end_style_properties(self):
		""" update self.tmp_instance_style, self.tmp_style_properties = [] """
		self.tmp_instance_style.add_props(self.tmp_style_properties)
		self.tmp_style_properties = []

	""" STYLES DES LISTES, NUMEROTATIONS ET INDENTATIONS """
	def init_style_tab_stop(self, attrs):
		dofct("init_style_tab_stop")

	def init_text_list_style (self, attrs):
		dofct("init_text_list_style")
		self.TEXT_LIST_STYLE = True
		self.tmp_nom_style = attrs.get('style:name')

	def end_text_list_style (self):
		self.TEXT_LIST_STYLE = True
		self.tmp_nom_style = None


	""" BALISES DE TEXTE  - TEXT TAGS """
	""" text:p, text:span, text:h, text:a"""
	def init_textp (self, attrs):
		self.TEXT_P = True
		self.tmp_textp_style = attrs.get("text:style-name")

	def end_textp (self):
		self.TEXT_P = False
		self.tmp_textp_style = None
		self.tmp_textfinal_style = oostyles.OOStyle()

	def init_textspan(self, attrs):
		self.TEXT_SPAN = True
		self.tmp_textspan_style = attrs.get("text:style-name")

	def end_textspan(self):
		self.TEXT_SPAN = False
		self.tmp_textspan_style = None
		self.tmp_textfinal_style = oostyles.OOStyle()

	def init_texth (self, attrs):
		self.TEXT_H = True
		self.tmp_texth_style = attrs.get("text:style-name")

	def end_texth (self):
		self.TEXT_H = False
		self.tmp_texth_style = None
		self.tmp_textfinal_style = oostyles.OOStyle()

	""" TAB, Line Break """

	def line_break (self):
		if self.FOOTNOTE : self.NOTE_CONTENT.texte (self.ui.linebrk)
		else : self.main.texte (self.ui.linebrk)






class Content (Styles):
	""" traitement du fichier content.xml et des objets liés"""
	def __init__(self, ui_name):
		Styles.__init__(self, ui_name)
		self.main = self.ui.ZoneTexte()
		self.LIST_ZONETEXTE.append(self.main)

	def characters (self, content):
		if self.TEXT_H : self.maj_style (self.tmp_texth_style)
		if self.TEXT_P : self.maj_style (self.tmp_textp_style)
		if self.TEXT_SPAN : self.maj_style (self.tmp_textspan_style)

		if self.main.is_new_paragraph() : self.main.texte (self.ui.lineret * (self.TEXT_LIST_ITEM-2) )

		if self.FOOTNOTE :
			if self.FOOTNOTE_CITATION :
				self.main.texte_oo(content, self.STYLE_NOTES_CONTENT)
				self.NOTE_CONTENT.texte_oo("["+content+"]"+self.ui.lineret, self.STYLE_NOTES)

			elif self.FOOTNOTE_BODY :
				self.NOTE_CONTENT.texte(content)

		else :
			if not self.TABLE : self.main.texte_oo(content, self.tmp_textfinal_style)
			else : self.tmp_cell.texte += content


	def startElement(self, name, attrs):
		# Définition des styles (de content.xml)
		if name=="office:automatic-styles" : self.AUTOMATIC_STYLES = True

		elif self.AUTOMATIC_STYLES :
			if name=="style:style" : self.init_style_decl(attrs)
			elif name=="text:list-style" : self.init_text_list_style(attrs)
			elif self.STYLE_STYLE :
				if name=="style:properties": self.init_style_properties(attrs)
				elif name in ("style:tab-stop", "style:tab-stops") : self.init_style_tab_stop(attrs)
				else : todo("Option de style",name, attrs.keys(), True)
			elif self.TEXT_LIST_STYLE : pass


		# corps du document (texte)
		elif name=="office:body" : self.BODY = True
		elif self.BODY and not self.TABLE:
			if name=="text:p" : self.init_textp(attrs)
			elif name=="text:h" : self.init_texth(attrs)
			elif name=="text:span" : self.init_textspan(attrs)
			elif name=='text:s' : self.main.texte(' '*int(attrs.get('text:c',True)))
			elif name=='text:tab-stop' : self.main.texte(self.ui.lineret)
			elif name=="text:ordered-list" : self.init_text_ordered_list(attrs)
			elif name=="text:unordered-list" : self.init_text_unordered_list(attrs)
			elif name=="text:list-item" : self.init_text_list_item(attrs)
			elif name=="draw:image" : self.aff_image(attrs)
			elif name=="draw:object" : self.aff_objet(attrs)
			elif name=="table:table" : self.init_table (attrs)
			elif name=="text:line-break" : self.line_break ()
			elif name=="text:footnote": self.FOOTNOTE=True
			elif name=="text:footnote-citation": self.FOOTNOTE_CITATION=True
			elif name=="text:footnote-body": self.init_footnote_body(attrs)
			else : todo("Balise", name, aff = True)

		elif self.BODY and self.TABLE :
			if name=="text:p" : self.init_textp(attrs)
			elif name=="text:span" : self.init_textspan(attrs)
			elif name=="table:table-column" : self.init_table_column (attrs)
			elif name=="table:table-header-rows" : self.init_table_header_rows (attrs)
			elif name=="table:table-row" : self.init_table_row (attrs)
			elif name=="table:table-cell" : self.init_table_cell (attrs)
			elif name=="draw:image" : self.aff_table_image(attrs)
			elif name=="draw:object" : self.aff_table_objet(attrs)

			else : todo("Balise", name, aff = True)

		else : todo("Balise", name, aff = True)


	def endElement(self, name):
		if not self.TABLE :
			if name=="office:automatic-styles" : self.AUTOMATIC_STYLES = False
			elif name=="office:body" : self.BODY = False
			elif name=="style:style" : self.end_style_decl()
			elif name=="style:properties": self.end_style_properties()
			elif name=="text:list-style" : self.end_text_list_style()
			elif name=="text:p" : self.end_textp()
			elif name=="text:span" : self.end_textspan()
			elif name=="text:h" : self.end_texth()
			elif name=="text:ordered-list" : self.end_text_ordered_list()
			elif name=="text:unordered-list" : self.end_text_unordered_list()
			elif name=="text:list-item" : self.end_text_list_item()
			elif name=="draw:image" : self.main.texte(self.ui.linebrk)#en attendant la gestion de l'adaptation de texte
			elif name=="text:footnote": self.FOOTNOTE=False
			elif name=="text:footnote-citation": self.FOOTNOTE_CITATION=False
			elif name=="text:footnote-body": self.end_footnote_body()
		elif self.TABLE :
			if name=="table:table" : self.end_table()
			elif name=="table:table-column" : self.end_table_column ()
			elif name=="table:table-row" : self.end_table_row ()
			elif name=="table:table-header-rows" : self.end_table_header_rows ()
			elif name=="table:table-cell" : self.end_table_cell ()
# 			elif name=="text:p" : self.end_table_textp()
			elif name=="text:span" : self.end_textspan()
			elif name=="draw:image" : pass #self.tmp_table_cell.texte(self.ui.linebrk)
			elif name=="draw:object" : pass #self.tmp_table_cell.texte(self.ui.linebrk)

	def skippedEntity(self, name): pass


	#################"

	def aff_image (self, attrs):
		src = attrs.get('xlink:href')[1:]
		try : self.main.image(attrs.get('xlink:href')[1:])
		except : self.main.texte('[%s]\n'%attrs.get('draw:name',''))

	def aff_table_image (self, attrs):
		todo("TODO", "aff_table_image", "oocontent", 1)
# 		try : self.tmp_table_cell.image(attrs.get('xlink:href')[1:])
# 		except : self.tmp_table_cell.texte('[%s]\n'%attrs.get('draw:name',''))

	def aff_objet_all (self, attrs):
		href = attrs.get("xlink:href")
		name = attrs.get("draw:style-name")
		object = ooobject.Object(href, self.ui.name)
		return object.OBJECT

	def aff_objet(self, attrs): self.main.objet(self.aff_objet_all (attrs))
	def aff_table_objet(self, attrs) :
		pass #self.tmp_table_cell.objet(self.aff_objet_all (attrs))


	###### methode de traitement #######
	###### treatment methods ##########

	def init_textp (self, attrs):
		Styles.init_textp(self, attrs)
		self.main.new_paragraph()

	def end_textp (self):
		Styles.end_textp(self)
		self.line_break ()

	def init_footnote_body(self, attrs):
		self.FOOTNOTE_BODY = True

	def end_footnote_body(self):
		self.FOOTNOTE_BODY = False
		self.NOTE_CONTENT.texte(2*self.ui.linebrk)

	"""     LIST     """

	"""
	self.TEXT_LIST_ITEM = False
	self.TEXT_LIST = False
	self.TEXT_ORDERED_LIST = 0
	self.TEXT_UNORDERED_LIST = 0
	self.tmp_list_style_name  =  None

	CODE SAMPLE
	<text:ordered-list text:style-name = "L1">
		<text:list-item>
			<text:p>This is the first list item</text:p>
		</text:list-item>
		<text:list-item>
			<text:p>This is the secont list item</text:p>
		</text:list-item>
	</text:ordered-list>
	"""

	def init_text_ordered_list(self, attrs) :
		attrs.get("text:style-name")
		self.LIST = True

	def end_text_ordered_list(self) :
		pass
		self.LIST = False

	def init_text_unordered_list(self, attrs) :
		attrs.get("text:style-name")
		self.LIST = True

	def end_text_unordered_list(self) :
		self.LIST = False
		pass

	def init_text_list_item(self, attrs) :
		self.TEXT_LIST_ITEM += 1
		pass


	def end_text_list_item(self) :
		self.TEXT_LIST_ITEM-=1
		if not self.TEXT_LIST_ITEM : pass


	""" TABLES """

	def init_table (self, attrs):
		pass
		self.TABLE = True
		self.TABLE_COLUMN = 0
		oostyle = self.load_style(attrs.get('table:style-name'))

		self.tmp_table = ootable.OOTable ()
		self.tmp_table.set_oostyle (oostyle)
		self.tmp_table.set_name (attrs.get('table:name'))


	def init_table_column (self, attrs) :
		oostyle = self.load_style(attrs.get('table:style-name'))

		repeat = attrs.get("table:number-columns-repeated")
		if repeat : repeat = int (repeat)
		else : repeat = 1

		for x in xrange (repeat) : self.tmp_table.add_col (oostyle)


	def init_table_header_rows (self, attrs) :
		pass
		self.TABLE_HEADER_ROWS = True

	def init_table_row (self, attrs) :
		pass
		self.tmp_table.add_row()


	def init_table_cell (self, attrs) :
		pass
		oostyle = self.load_style(attrs.get('table:style-name'))

		self.tmp_cell = oocell.OOCell ()
		self.tmp_cell.set_oostyle(oostyle)



	def end_table (self):
		pass
		self.tmp_table.end_table()
		self.ui_table = self.ui.Table(self.tmp_table)
		self.main.objet(self.ui_table)

		self.tmp_table = ootable.OOTable ()
		self.ui_table = self.ui.Table(self.tmp_table)

		self.TABLE = False


	def end_table_column (self) : pass

	def end_table_header_rows (self) :
		pass
		self.TABLE_HEADER_ROWS = False

	def end_table_row (self) : pass

	def end_table_cell (self) :
		pass
		self.tmp_table.add_cell(self.tmp_cell)
		self.tmp_cell = oocell.OOCell()






	""" Traitement en début et fin de lecture de documents
		Begin an end document treatment """

	def startDocument (self):
		if self.verbose:
			print _("Start reading document  %s") % ( 'content.xml', )

		if self.HEADER_CONTENT.not_empty():
			self.main.texte_ui("%s : " % _("Header"), self.ui.convert("fo:font-weight","bold"))
			self.main.objet(self.HEADER_CONTENT)

	def endDocument (self):
		if '-d' in sys.argv : all_todo ()
		print sys.argv
		if self.verbose :
			print "\n"
			print _("End reading document %s") % ( 'content.xml', )

		if self.FOOTER_CONTENT.not_empty():
			self.main.texte_ui("\n%s : " % _("Footer"), self.ui.convert("fo:font-weight","bold"))
			self.main.objet(self.FOOTER_CONTENT)
